extends KinematicBody2D

var MAX_SPEED = 600
var ACCELERATION = 1800
var FRICTION = 2000
var MAX_LIFE = 30
const PICKUP_BONUS = 5
export var rota_speed = 100
export var slow_cooldown = 5
export var fast_cooldown = 5

var slow_trail = load("res://Scene/SlowPaint.tscn")
var fast_trail = load("res://Scene/FastPaint.tscn")

var slow_cooldown_elapsed = 5
var fast_cooldown_elapsed = 5

var follow1;
var follow2;

var vel = Vector2.ZERO
var last_dir = Vector2.ZERO
var curLife = MAX_LIFE

var coroutines = []

enum EInput{
	UP,
	RIGHT,
	DOWN,
	LEFT
}

const baseInputs = ["ui_up", "ui_right", "ui_down", "ui_left"]
var inputs = ["ui_up", "ui_right", "ui_down", "ui_left"]

onready var animationPlayer = $AnimationPlayer
onready var animationTree = $AnimationTree
onready var animationState = animationTree.get("parameters/playback")
onready var pickupPlayer = $PickupPlayer
var healthBar = null

func _ready():
	get_node("ConfusionStars").visible = false
	follow1 = get_node("ConfusionStars/Path2D/PathFollow2D")
	follow2 = get_node("ConfusionStars/Path2D/PathFollow2D2")	
	if self.name == "Player1":
		healthBar = get_tree().get_nodes_in_group("healthbar")[0]
	elif self.name == "Player2":
		healthBar = get_tree().get_nodes_in_group("healthbar")[1]
	healthBar.maxHealthUpdated(MAX_LIFE)

func launch_slow():
	if slow_cooldown_elapsed < slow_cooldown:
		return
	slow_cooldown_elapsed = 0
	var world = get_parent()
	var inst = load("res://Scene/SlowTrail.tscn").instance()
	var dir = get_dir()	
	if dir == Vector2.ZERO:
		dir = (last_dir if last_dir != Vector2.ZERO else Vector2.DOWN)
	var angle = acos(dir.dot(Vector2.RIGHT))
	var modifier = 0
	if dir.y < 0:
		modifier += angle * -2
	inst.transform = inst.transform.rotated(angle + modifier)
	inst.position = self.position
	inst.slowFactor = 2
	world.add_child(inst)
	
func launch_fast():
	if fast_cooldown_elapsed < fast_cooldown:
		return
	fast_cooldown_elapsed = 0
	var world = get_parent()
	var inst = load("res://Scene/FastTrail.tscn").instance()
	var dir = get_dir()	
	if dir == Vector2.ZERO:
		dir = (last_dir if last_dir != Vector2.ZERO else Vector2.DOWN)
	var angle = acos(dir.dot(Vector2.RIGHT))
	var modifier = 0
	if dir.y < 0:
		modifier += angle * -2
	inst.transform = inst.transform.rotated(angle + modifier)
	inst.position = self.position
	inst.fastFactor = 2
	world.add_child(inst)

func _process(delta):
	follow1.set_offset(follow1.get_offset() + delta * rota_speed)
	follow2.set_offset(follow2.get_offset() + delta * rota_speed)
	
	if (self.name == "Player1" && Input.get_action_strength("gameplay_slow") > 0) \
	   || (self.name == "Player2" && Input.get_action_strength("gameplay_slow2") > 0):
		launch_slow()
		
	if (self.name == "Player1" && Input.get_action_strength("gameplay_fast") > 0) \
	   || (self.name == "Player2" && Input.get_action_strength("gameplay_fast2") > 0):
		launch_fast()
	
	updatePaintEffects(delta)
	curLife -= delta
	slow_cooldown_elapsed += delta
	if curLife <= 0:
		get_node("/root/World/GameManager").endGame()
	healthBar.healthUpdated(curLife)		

func get_dir():
	var input_vector = Vector2.ZERO
	
	if self.name == "Player1":
		input_vector.x = Input.get_action_strength(inputs[EInput.RIGHT]) - Input.get_action_strength(inputs[EInput.LEFT])
		input_vector.y = Input.get_action_strength(inputs[EInput.DOWN]) - Input.get_action_strength(inputs[EInput.UP])
	elif self.name == "Player2":
		input_vector.x = Input.get_action_strength(inputs[EInput.RIGHT] + "2") - Input.get_action_strength(inputs[EInput.LEFT] + "2")
		input_vector.y = Input.get_action_strength(inputs[EInput.DOWN] + "2") - Input.get_action_strength(inputs[EInput.UP] + "2")
	
	input_vector = input_vector.normalized()
	return input_vector

func _physics_process(delta):
	var input_vector = get_dir()
	
	if input_vector != Vector2.ZERO:
		last_dir = input_vector
		animationTree.set("parameters/Idle/blend_position", input_vector)
		animationTree.set("parameters/Run/blend_position", input_vector)
		animationState.travel("Run")
		vel = vel.move_toward(input_vector * MAX_SPEED, ACCELERATION * delta)
	else:
		animationState.travel("Idle")
		vel = vel.move_toward(Vector2.ZERO, FRICTION * delta)
	
	vel = move_and_slide(vel)

func updatePaintEffects(delta):	
	var updated = []	
	for rout in coroutines:				
		if rout != null && rout.is_valid():
			var new_rout = rout.resume(delta)		
			if new_rout != null && new_rout.is_valid():
				updated.append(new_rout)				
		coroutines.remove(coroutines.find(rout))
						
	for rout in updated:
		coroutines.append(rout)

func addCoroutine(routine):
	pickupPlayer.play()
	curLife = MAX_LIFE if (curLife + PICKUP_BONUS > MAX_LIFE) else curLife + PICKUP_BONUS
	coroutines.append(routine)
